"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.None = exports.Some = exports.isOption = void 0;
const result_1 = require("./result");
const IsSome = Symbol("IsSome");
class OptionType {
    constructor(val, some) {
        this.val = val;
        this[IsSome] = some;
        Object.freeze(this);
    }
    /**
     * See `like()`.
     * @deprecated
     */
    is(cmp) {
        return this.isLike(cmp);
    }
    /**
     * Compares the Option to `cmp`, returns true if both are `Some` or both
     * are `None`. Also acts as a type guard for `Option<unknown>`.
     *
     * ```
     * const s: Option<number> = Some(1);
     * const n: Option<number> = None;
     *
     * assert.equal(s.isLike(Some(10)), true);
     * assert.equal(n.isLike(None), true);
     * assert.equal(s.isLike(n), false);
     * ```
     */
    isLike(cmp) {
        return cmp instanceof OptionType && this[IsSome] === cmp[IsSome];
    }
    /**
     * See `equals()`.
     * @deprecated
     */
    eq(cmp) {
        return this.equals(cmp);
    }
    /**
     * Compares the Option to `cmp` for equality. Returns `true` when both are
     * `Some` with identical contained values, or both are `None`.
     *
     * ```
     * const val = { x: 10 };
     * const s: Option<{ x: number; }> = Some(val);
     * const n: Option<{ x: number; }> = None;
     *
     * assert.equal(s.equals(Some(val)), true);
     * assert.equal(n.equals(None), true):
     * assert.equal(s.equals(Some({ x: 10 })), false);
     * assert.equal(s.equals(n), false);
     * ```
     */
    equals(cmp) {
        return this[IsSome] === cmp[IsSome] && this.val === cmp.val;
    }
    /**
     * Compares the Option to `cmp` for inequality. Returns true when both are
     * different types (`Some`/`None`) or their contained values are not
     * identical (`!==`).
     *
     * const val = { x: 10 };
     * const s: Option<{ x: number; }> = Some(val);
     * const n: Option<{ x: number; }> = None;
     *
     * assert.equal(s.neq(Some(val)), false);
     * assert.equal(n.neq(None), false);
     * assert.equal(s.neq(Some({ x: 10})), true);
     * assert.equal(s.new(n), true);
     * ```
     *
     * @deprecated
     */
    neq(cmp) {
        return this[IsSome] !== cmp[IsSome] || this.val !== cmp.val;
    }
    /**
     * Returns true if the Option is `Some`. Acts as a type guard for
     * `this is Some<T>`.
     *
     * ```
     * const x = Some(10);
     * assert.equal(x.isSome(), true);
     *
     * const x: Option<number> = None;
     * assert.equal(x.isSome(), false);
     * ```
     */
    isSome() {
        return this[IsSome];
    }
    /**
     * Returns true if the Option is `None`. Acts as a type guard for
     * `this is None<never>`.
     *
     * ```
     * const x = Some(10);
     * assert.equal(x.isNone(), false);
     *
     * const x: Option<number> = None;
     * assert.equal(x.isNone(), true);
     * ```
     */
    isNone() {
        return !this[IsSome];
    }
    /**
    Returns the contained `Some` value and throws `Error(msg)` if `None`.
 
    To avoid throwing, consider `isSome`, `unwrapOr`, `unwrapOrElse` or
    `match` to handle the `None` case.
    
    ```
    const x = Some(1);
    assert.equal(x.expect("Is empty"), 1);
    
    const x: Option<number> = None;
    const y = x.expect("Is empty"); // throws
    * ```
    */
    expect(msg) {
        if (this[IsSome]) {
            return this.val;
        }
        else {
            throw new Error(msg);
        }
    }
    /**
    Returns the contained `Some` value and throws if `None`.
 
    To avoid throwing, consider `isSome`, `unwrapOr`, `unwrapOrElse` or
    `match` to handle the `None` case. To throw a more informative error use
    `expect`.
    
    ```
    const x = Some(1);
    assert.equal(x.unwrap(), 1);
    
    const x: Option<number> = None;
    const y = x.unwrap(); // throws
    * ```
    */
    unwrap() {
        return this.expect("Failed to unwrap Option (found None)");
    }
    /**
     * Returns the contained `Some` value or a provided default.
     *
     * The provided default is eagerly evaluated. If you are passing the result
     * of a function call, consider `unwrapOrElse`, which is lazily evaluated.
     *
     * ```
     * const x = Some(10);
     * assert.equal(x.unwrapOr(1), 10);
     *
     * const x: Option<number> = None;
     * assert.equal(x.unwrapOr(1), 1);
     * ```
     */
    unwrapOr(def) {
        return this[IsSome] ? this.val : def;
    }
    /**
     * Returns the contained `Some` value or computes it from a function.
     *
     * ```
     * const x = Some(10);
     * assert.equal(x.unwrapOrElse(() => 1 + 1), 10);
     *
     * const x: Option<number> = None;
     * assert.equal(x.unwrapOrElse(() => 1 + 1), 2);
     * ```
     */
    unwrapOrElse(f) {
        return this[IsSome] ? this.val : f();
    }
    /**
     * Returns the contained `Some` value or undefined if `None`.
     *
     * Most problems are better solved using one of the other `unwrap_` methods.
     * This method should only be used when you are certain that you need it.
     *
     * ```
     * const x = Some(10);
     * assert.equal(x.unwrapUnchecked(), 10);
     *
     * const x: Option<number> = None;
     * assert.equal(x.unwrapUnchecked(), undefined);
     * ```
     */
    unwrapUnchecked() {
        return this.val;
    }
    /**
     * Returns the Option if it is `Some`, otherwise returns `optb`.
     *
     * `optb` is eagerly evaluated. If you are passing the result of a function
     * call, consider `orElse`, which is lazily evaluated.
     *
     * ```
     * const x = Some(10);
     * const xor = x.or(Some(1));
     * assert.equal(xor.unwrap(), 10);
     *
     * const x: Option<number> = None;
     * const xor = x.or(Some(1));
     * assert.equal(xor.unwrap(), 1);
     * ```
     */
    or(optb) {
        return this[IsSome] ? this : optb;
    }
    /**
     * Returns the Option if it is `Some`, otherwise returns the value of `f()`.
     *
     * ```
     * const x = Some(10);
     * const xor = x.orElse(() => Some(1));
     * assert.equal(xor.unwrap(), 10);
     *
     * const x: Option<number> = None;
     * const xor = x.orElse(() => Some(1));
     * assert.equal(xor.unwrap(), 1);
     * ```
     */
    orElse(f) {
        return this[IsSome] ? this : f();
    }
    /**
     * Returns `None` if the Option is `None`, otherwise returns `optb`.
     *
     * ```
     * const x = Some(10);
     * const xand = x.and(Some(1));
     * assert.equal(xand.unwrap(), 1);
     *
     * const x: Option<number> = None;
     * const xand = x.and(Some(1));
     * assert.equal(xand.isNone(), true);
     *
     * const x = Some(10);
     * const xand = x.and(None);
     * assert.equal(xand.isNone(), true);
     * ```
     */
    and(optb) {
        return this[IsSome] ? optb : exports.None;
    }
    /**
     * Returns `None` if the option is `None`, otherwise calls `f` with the
     * `Some` value and returns the result.
     *
     * ```
     * const x = Some(10);
     * const xand = x.andThen((n) => n + 1);
     * assert.equal(xand.unwrap(), 11);
     *
     * const x: Option<number> = None;
     * const xand = x.andThen((n) => n + 1);
     * assert.equal(xand.isNone(), true);
     *
     * const x = Some(10);
     * const xand = x.andThen(() => None);
     * assert.equal(xand.isNone(), true);
     * ```
     */
    andThen(f) {
        return this[IsSome] ? f(this.val) : exports.None;
    }
    /**
     * Maps an `Option<T>` to `Option<U>` by applying a function to the `Some`
     * value.
     *
     * ```
     * const x = Some(10);
     * const xmap = x.map((n) => `number ${n}`);
     * assert.equal(xmap.unwrap(), "number 10");
     * ```
     */
    map(f) {
        return this[IsSome] ? new OptionType(f(this.val), true) : exports.None;
    }
    /**
     * Returns the provided default if `None`, otherwise calls `f` with the
     * `Some` value and returns the result.
     *
     * The provided default is eagerly evaluated. If you are passing the result
     * of a function call, consider `mapOrElse`, which is lazily evaluated.
     *
     * ```
     * const x = Some(10);
     * const xmap = x.mapOr(1, (n) => n + 1);
     * assert.equal(xmap.unwrap(), 11);
     *
     * const x: Option<number> = None;
     * const xmap = x.mapOr(1, (n) => n + 1);
     * assert.equal(xmap.unwrap(), 1);
     * ```
     */
    mapOr(def, f) {
        return this[IsSome] ? f(this.val) : def;
    }
    /**
     * Computes a default return value if `None`, otherwise calls `f` with the
     * `Some` value and returns the result.
     *
     * const x = Some(10);
     * const xmap = x.mapOrElse(() => 1 + 1, (n) => n + 1);
     * assert.equal(xmap.unwrap(), 11);
     *
     * const x: Option<number> = None;
     * const xmap = x.mapOrElse(() => 1 + 1, (n) => n + 1);
     * assert.equal(xmap.unwrap(), 2);
     * ```
     */
    mapOrElse(def, f) {
        return this[IsSome] ? f(this.val) : def();
    }
    /**
     * Transforms the `Option<T>` into a `Result<T, E>`, mapping `Some(v)` to
     * `Ok(v)` and `None` to `Err(err)`.
     *
     * ```
     * const x = Some(10);
     * const res = x.okOr("Is empty");
     * assert.equal(x.isOk(), true);
     * assert.equal(x.unwrap(), 10);
     *
     * const x: Option<number> = None;
     * const res = x.okOr("Is empty");
     * assert.equal(x.isErr(), true);
     * assert.equal(x.unwrap_err(), "Is empty");
     * ```
     */
    okOr(err) {
        return this[IsSome] ? (0, result_1.Ok)(this.val) : (0, result_1.Err)(err);
    }
    /**
     * Transforms the `Option<T>` into a `Result<T, E>`, mapping `Some(v)` to
     * `Ok(v)` and `None` to `Err(f())`.
     *
     * ```
     * const x = Some(10);
     * const res = x.okOrElse(() => ["Is", "empty"].join(" "));
     * assert.equal(x.isOk(), true);
     * assert.equal(x.unwrap(), 10);
     *
     * const x: Option<number> = None;
     * const res = x.okOrElse(() => ["Is", "empty"].join(" "));
     * assert.equal(x.isErr(), true);
     * assert.equal(x.unwrap_err(), "Is empty");
     * ```
     */
    okOrElse(f) {
        return this[IsSome] ? (0, result_1.Ok)(this.val) : (0, result_1.Err)(f());
    }
}
/**
 * Tests the provided `val` is an Option. Acts as a type guard for
 * `val is Option<unknown>`.
 *
 * ```
 * assert.equal(Option.is(Some(1), true);
 * assert.equal(Option.is(None, true));
 * assert.equal(Option.is(Ok(1), false));
 * ```
 */
function isOption(val) {
    return val instanceof OptionType;
}
exports.isOption = isOption;
/**
 * Creates a `Some<T>` value, which can be used where an `Option<T>` is
 * required. See Option for more examples.
 *
 * ```
 * const x = Some(10);
 * assert.equal(x.isSome(), true);
 * assert.equal(x.unwrap(), 10);
 * ```
 */
function Some(val) {
    return new OptionType(val, true);
}
exports.Some = Some;
/**
 * The `None` value, which can be used where an `Option<T>` is required.
 * See Option for more examples.
 *
 * ```
 * const x = None;
 * assert.equal(x.isNone(), true);
 * const y = x.unwrap(); // throws
 * ```
 */
exports.None = new OptionType(undefined, false);
Object.assign(OptionType.prototype, {
    is_some: OptionType.prototype.isSome,
    is_none: OptionType.prototype.isNone,
    unwrap_or: OptionType.prototype.unwrapOr,
    unwrap_or_else: OptionType.prototype.unwrapOrElse,
    unwrap_unchecked: OptionType.prototype.unwrapUnchecked,
    or_else: OptionType.prototype.orElse,
    and_then: OptionType.prototype.andThen,
    map_or: OptionType.prototype.mapOr,
    map_or_else: OptionType.prototype.mapOrElse,
    ok_or: OptionType.prototype.okOr,
    ok_or_else: OptionType.prototype.okOrElse,
});
Object.freeze(OptionType.prototype);
Object.freeze(isOption);
Object.freeze(Some);
Object.freeze(exports.None);
