import { Option } from "./option";
declare const IsOk: unique symbol;
export declare type Result<T, E> = Ok<T, E> | Err<E, T>;
export declare type Ok<T, E> = ResultType<T, E> & {
    [IsOk]: true;
};
export declare type Err<E, T> = ResultType<T, E> & {
    [IsOk]: false;
};
declare class ResultType<T, E> {
    private val;
    readonly [IsOk]: boolean;
    constructor(val: T | E, ok: boolean);
    /**
     * See `isLike()`.
     * @deprecated
     */
    is(cmp: unknown): cmp is Result<unknown, unknown>;
    /**
     * Compares the Result to `cmp`, returns true if both are `Ok` or both
     * are `Err`. Acts as a type guard for `cmp is Result<unknown, unknown>`.
     *
     * ```
     * const o = Ok(1);
     * const e = Err(1);
     *
     * assert.equal(o.isLike(Ok(1))), true);
     * assert.equal(e.isLike(Err(1)), true);
     * assert.equal(o.isLike(e), false);
     * ```
     */
    isLike(cmp: unknown): cmp is Result<unknown, unknown>;
    /**
     * See `equals()`.
     * @deprecated
     */
    eq(cmp: Result<T, E>): boolean;
    /**
     * Compares the Result to `cmp` for equality. Returns `true` when both are
     * the same type (`Ok`/`Err`) and their contained values are identical
     * (`===`).
     *
     * ```
     * const val = { x: 10 };
     * const o: Result<{ x: number; }, { x: number; }> = Ok(val);
     * const e: Result<{ x: number; }, { x: number; }> = Err(val);
     *
     * assert.equal(o.equals(Ok(val)), true);
     * assert.equal(e.equals(Err(val)), true):
     * assert.equal(o.equals(Ok({ x: 10 })), false);
     * assert.equal(e.equals(Err({ x: 10 })), false);
     * assert.equal(o.equals(e), false);
     * ```
     */
    equals(cmp: Result<T, E>): boolean;
    /**
     * Compares the Result to `cmp` for inequality. Returns true when both are
     * different types (`Ok`/`Err`) or their contained values are not identical
     * (`!==`).
     *
     * ```
     * const val = { x: 10 };
     * const o: Result<{ x: number; }, { x: number; }> = Ok(val);
     * const e: Result<{ x: number; }, { x: number; }> = Err(val);
     *
     * assert.equal(o.neq(Ok(val)), false);
     * assert.equal(e.neq(Err(val)), false):
     * assert.equal(o.neq(Ok({ x: 10 })), true);
     * assert.equal(e.neq(Err({ x: 10 })), true);
     * assert.equal(o.neq(e), true);
     * ```
     *
     * @deprecated
     */
    neq(cmp: Result<T, E>): boolean;
    /**
     * Returns true if the Result is `Ok`. Acts as a type guard for
     * `this is Ok<T, any>`.
     *
     * @example
     * const x = Ok(10);
     * assert.equal(x.isOk(), true);
     *
     * const x = Err(10);
     * assert.equal(x.isOk(), false);
     */
    isOk(): this is Ok<T, any>;
    /**
     * Returns true if the Result is `Err`. Acts as a type guard for
     * `this is Err<E, any>`.
     *
     * @example
     * const x = Ok(10);
     * assert.equal(x.isErr(), false);
     *
     * const x = Err(10);
     * assert.equal(x.isErr(), true);
     */
    isErr(): this is Err<E, any>;
    /**
    Returns the contained `Ok` value and throws `Error(msg)` if `Err`.
 
    To avoid throwing, consider `isOk`, `unwrapOr`, `unwrapOrElse` or
    `match` to handle the `Err` case.
    
    @example
    const x = Ok(1);
    assert.equal(x.expect("Was Err"), 1);
    
    const x = Err(1);
    const y = x.expect("Was Err"); // throws
    */
    expect(msg: string): T;
    /**
    Returns the contained `Err` value and throws `Error(msg)` if `Ok`.
 
    To avoid throwing, consider `isErr` or `match` to handle the `Ok` case.
    
    @example
    const x = Ok(1);
    const y = x.expectErr("Was Ok"); // throws
 
    const x = Err(1);
    assert.equal(x.expectErr("Was Ok"), 1);
    */
    expectErr(msg: string): E;
    /**
    Returns the contained `Ok` value and throws if `Err`.
 
    To avoid throwing, consider `isOk`, `unwrapOr`, `unwrapOrElse` or
    `match` to handle the `Err` case. To throw a more informative error use
    `expect`.
    
    @example
    const x = Ok(1);
    assert.equal(x.unwrap(), 1);
    
    const x = Err(1);
    const y = x.unwrap(); // throws
    */
    unwrap(): T;
    /**
    Returns the contained `Err` value and throws if `Ok`.
 
    To avoid throwing, consider `isErr` or `match` to handle the `Ok` case.
    To throw a more informative error use `expectErr`.
    
    @example
    const x = Ok(1);
    const y = x.unwrap(); // throws
    
    const x = Err(1);
    assert.equal(x.unwrap(), 1);
    */
    unwrapErr(): E;
    /**
     * Returns the contained `Ok` value or a provided default.
     *
     * The provided default is eagerly evaluated. If you are passing the result
     * of a function call, consider `unwrapOrElse`, which is lazily evaluated.
     *
     * @example
     * const x = Ok(10);
     * assert.equal(x.unwrapOr(1), 10);
     *
     * const x = Err(10);
     * assert.equal(x.unwrapOr(1), 1);
     */
    unwrapOr(def: T): T;
    /**
     * Returns the contained `Ok` value or computes it from a function.
     *
     * @example
     * const x = Ok(10);
     * assert.equal(x.unwrapOrElse(() => 1 + 1), 10);
     *
     * const x = Err(10);
     * assert.equal(x.unwrapOrElse(() => 1 + 1), 2);
     */
    unwrapOrElse(f: () => T): T;
    /**
     * Returns the contained `Ok` or `Err` value.
     *
     * Most problems are better solved using one of the other `unwrap_` methods.
     * This method should only be used when you are certain that you need it.
     *
     * @example
     * const x = Ok(10);
     * assert.equal(x.unwrapUnchecked(), 10);
     *
     * const x = Err(20);
     * assert.equal(x.unwrapUnchecked(), 20);
     */
    unwrapUnchecked(): T | E;
    /**
     * Returns the Option if it is `Ok`, otherwise returns `resb`.
     *
     * `resb` is eagerly evaluated. If you are passing the result of a function
     * call, consider `orElse`, which is lazily evaluated.
     *
     * @example
     * const x = Ok(10);
     * const xor = x.or(Ok(1));
     * assert.equal(xor.unwrap(), 10);
     *
     * const x = Err(10);
     * const xor = x.or(Ok(1));
     * assert.equal(xor.unwrap(), 1);
     */
    or(resb: Result<T, E>): Result<T, E>;
    /**
     * Returns the Result if it is `Ok`, otherwise returns the value of `f()`
     * mapping `Result<T, E>` to `Result<T, F>`.
     *
     * @example
     * const x = Ok(10);
     * const xor = x.orElse(() => Ok(1));
     * assert.equal(xor.unwrap(), 10);
     *
     * const x = Err(10);
     * const xor = x.orElse(() => Ok(1));
     * assert.equal(xor.unwrap(), 1);
     *
     * const x = Err(10);
     * const xor = x.orElse((e) => Err(`val ${e}`));
     * assert.equal(xor.unwrapErr(), "val 10");
     */
    orElse<F>(f: (err: E) => Result<T, F>): Result<T, F>;
    /**
     * Returns itself if the Result is `Err`, otherwise returns `resb`.
     *
     * @example
     * const x = Ok(10);
     * const xand = x.and(Ok(1));
     * assert.equal(xand.unwrap(), 1);
     *
     * const x = Err(10);
     * const xand = x.and(Ok(1));
     * assert.equal(xand.unwrapErr(), 10);
     *
     * const x = Ok(10);
     * const xand = x.and(Err(1));
     * assert.equal(xand.unwrapErr(), 1);
     */
    and<U>(resb: Result<U, E>): Result<U, E>;
    /**
     * Returns itself if the Result is `Err`, otherwise calls `f` with the `Ok`
     * value and returns the result.
     *
     * @example
     * const x = Ok(10);
     * const xand = x.andThen((n) => n + 1);
     * assert.equal(xand.unwrap(), 11);
     *
     * const x = Err(10);
     * const xand = x.andThen((n) => n + 1);
     * assert.equal(xand.unwrapErr(), 10);
     *
     * const x = Ok(10);
     * const xand = x.and(Err(1));
     * assert.equal(xand.unwrapErr(), 1);
     */
    andThen<U>(f: (val: T) => Result<U, E>): Result<U, E>;
    /**
     * Maps a `Result<T, E>` to `Result<U, E>` by applying a function to the
     * `Ok` value.
     *
     * @example
     * const x = Ok(10);
     * const xmap = x.map((n) => `number ${n}`);
     * assert.equal(xmap.unwrap(), "number 10");
     */
    map<U>(f: (val: T) => U): Result<U, E>;
    /**
     * Maps a `Result<T, E>` to `Result<T, F>` by applying a function to the
     * `Err` value.
     *
     * @example
     * const x = Err(10);
     * const xmap = x.mapErr((n) => `number ${n}`);
     * assert.equal(xmap.unwrapErr(), "number 10");
     */
    mapErr<F>(op: (err: E) => F): Result<T, F>;
    /**
     * Returns the provided default if `Err`, otherwise calls `f` with the
     * `Ok` value and returns the result.
     *
     * The provided default is eagerly evaluated. If you are passing the result
     * of a function call, consider `mapOrElse`, which is lazily evaluated.
     *
     * @example
     * const x = Ok(10);
     * const xmap = x.mapOr(1, (n) => n + 1);
     * assert.equal(xmap.unwrap(), 11);
     *
     * const x = Err(10);
     * const xmap = x.mapOr(1, (n) => n + 1);
     * assert.equal(xmap.unwrap(), 1);
     */
    mapOr<U>(def: U, f: (val: T) => U): U;
    /**
     * Computes a default return value if `Err`, otherwise calls `f` with the
     * `Ok` value and returns the result.
     *
     * const x = Ok(10);
     * const xmap = x.mapOrElse(() => 1 + 1, (n) => n + 1);
     * assert.equal(xmap.unwrap(), 11);
     *
     * const x = Err(10);
     * const xmap = x.mapOrElse(() => 1 + 1, (n) => n + 1);
     * assert.equal(xmap.unwrap(), 2);
     */
    mapOrElse<U>(def: (err: E) => U, f: (val: T) => U): U;
    /**
     * Transforms the `Result<T, E>` into an `Option<T>`, mapping `Ok(v)` to
     * `Some(v)`, discarding any `Err` value and mapping to None.
     *
     * @example
     * const x = Ok(10);
     * const opt = x.ok();
     * assert.equal(x.isSome(), true);
     * assert.equal(x.unwrap(), 10);
     *
     * const x = Err(10);
     * const opt = x.ok();
     * assert.equal(x.isNone(), true);
     * const y = x.unwrap(); // throws
     */
    ok(): Option<T>;
}
/**
 * Tests the provided `val` is an Result. Acts as a type guard for
 * `val is Result<unknown, unknown>`.
 *
 * @example
 * assert.equal(Result.is(Ok(1), true);
 * assert.equal(Result.is(Err(1), true));
 * assert.equal(Result.is(Some(1), false));
 */
export declare function isResult(val: unknown): val is Result<unknown, unknown>;
/**
 * Creates an `Ok<T, E>` value, which can be used where a `Result<T, E>` is
 * required. See Result for more examples.
 *
 * Note that the counterpart `Err` type `E` is set to the same type as `T`
 * by default. TypeScript will usually infer the correct `E` type from the
 * context (e.g. a function which accepts or returns a Result).
 *
 * @example
 * const x = Ok(10);
 * assert.equal(x.isSome(), true);
 * assert.equal(x.unwrap(), 10);
 */
export declare function Ok<T, E = T>(val: T): Ok<T, E>;
/**
 * Creates an `Err<E, T>` value, which can be used where a `Result<T, E>` is
 * required. See Result for more examples.
 *
 * Note that the counterpart `Ok` type `T` is set to the same type as `E`
 * by default. TypeScript will usually infer the correct `T` type from the
 * context (e.g. a function which accepts or returns a Result).
 *
 * @example
 * const x = Err(10);
 * assert.equal(x.isErr(), true);
 * assert.equal(x.unwrapErr(), 10);
 */
export declare function Err<E, T = E>(val: E): Err<E, T>;
interface ResultType<T, E> {
    /** @deprecated */
    is_ok: ResultType<T, E>["isOk"];
    /** @deprecated */
    is_err: ResultType<T, E>["isErr"];
    /** @deprecated */
    expect_err: ResultType<T, E>["expectErr"];
    /** @deprecated */
    unwrap_err: ResultType<T, E>["unwrapErr"];
    /** @deprecated */
    unwrap_or: ResultType<T, E>["unwrapOr"];
    /** @deprecated */
    unwrap_or_else: ResultType<T, E>["unwrapOrElse"];
    /** @deprecated */
    unwrap_unchecked: ResultType<T, E>["unwrapUnchecked"];
    /** @deprecated */
    or_else: ResultType<T, E>["orElse"];
    /** @deprecated */
    and_then: ResultType<T, E>["andThen"];
    /** @deprecated */
    map_err: ResultType<T, E>["mapErr"];
    /** @deprecated */
    map_or: ResultType<T, E>["mapOr"];
    /** @deprecated */
    map_or_else: ResultType<T, E>["mapOrElse"];
}
export {};
